//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
// Module to store twiddle factors
// in format Q9.9
//

module trom (
  clk_i, rst_i, adr_i,
  twr_o, twi_o
  );
  input clk_i;                    // Input clock signal
  input rst_i;                    // Input reset signal
  input [5:0] adr_i;              // Address of twiddle to be read
  output [17:0] twr_o;            // Output real part twiddle
  output [17:0] twi_o;            // Output imaginary part of twiddle
  
  reg [17:0] twr_o;
  reg [17:0] twi_o;
  
  // Internal registers and wires
  
  wire [17:0] sinx [63:0];   // Twiddle factors are 14-bit 2-s complement numbers sign extended to 18 bit
  wire [17:0] cosx [63:0];

  assign sinx[0] = 18'h0;        // 18'h0
  assign sinx[1] = 18'h3FFE7;
  assign sinx[2] = 18'h3FFCE;
  assign sinx[3] = 18'h3FFB5;
  assign sinx[4] = 18'h3FF9C;
  assign sinx[5] = 18'h3FF84;
  assign sinx[6] = 18'h3FF6B;
  assign sinx[7] = 18'h3FF54;
  assign sinx[8] = 18'h3FF3C;
  assign sinx[9] = 18'h3FF25;
  assign sinx[10] = 18'h3FF0F;
  assign sinx[11] = 18'h3FEF9;
  assign sinx[12] = 18'h3FEE4;
  assign sinx[13] = 18'h3FECF;
  assign sinx[14] = 18'h3FEBB;
  assign sinx[15] = 18'h3FEA8;
  assign sinx[16] = 18'h3FE96;
  assign sinx[17] = 18'h3FE85;
  assign sinx[18] = 18'h3FE74;
  assign sinx[19] = 18'h3FE65;
  assign sinx[20] = 18'h3FE56;
  assign sinx[21] = 18'h3FE49;
  assign sinx[22] = 18'h3FE3C;
  assign sinx[23] = 18'h3FE31;
  assign sinx[24] = 18'h3FE27;
  assign sinx[25] = 18'h3FE1E;
  assign sinx[26] = 18'h3FE16;
  assign sinx[27] = 18'h3FE0F;
  assign sinx[28] = 18'h3FE0A;
  assign sinx[29] = 18'h3FE06;
  assign sinx[30] = 18'h3FE02;
  assign sinx[31] = 18'h3FE01;
  assign sinx[32] = 18'h3FE00;
  assign sinx[33] = 18'h3FE01;
  assign sinx[34] = 18'h3FE02;
  assign sinx[35] = 18'h3FE06;
  assign sinx[36] = 18'h3FE0A;
  assign sinx[37] = 18'h3FE0F;
  assign sinx[38] = 18'h3FE16;
  assign sinx[39] = 18'h3FE1E;
  assign sinx[40] = 18'h3FE27;
  assign sinx[41] = 18'h3FE31;
  assign sinx[42] = 18'h3FE3C;
  assign sinx[43] = 18'h3FE49;
  assign sinx[44] = 18'h3FE56;
  assign sinx[45] = 18'h3FE65;
  assign sinx[46] = 18'h3FE74;
  assign sinx[47] = 18'h3FE85;
  assign sinx[48] = 18'h3FE96;
  assign sinx[49] = 18'h3FEA8;
  assign sinx[50] = 18'h3FEBB;
  assign sinx[51] = 18'h3FECF;
  assign sinx[52] = 18'h3FEE4;
  assign sinx[53] = 18'h3FEF9;
  assign sinx[54] = 18'h3FF0F;
  assign sinx[55] = 18'h3FF25;
  assign sinx[56] = 18'h3FF3C;
  assign sinx[57] = 18'h3FF54;
  assign sinx[58] = 18'h3FF6B;
  assign sinx[59] = 18'h3FF84;
  assign sinx[60] = 18'h3FF9C;
  assign sinx[61] = 18'h3FFB5;
  assign sinx[62] = 18'h3FFCE;
  assign sinx[63] = 18'h3FFE7;
  
  assign cosx[0] = 18'h00200;        //18'h00200
  assign cosx[1] = 18'h001FF;
  assign cosx[2] = 18'h001FE;
  assign cosx[3] = 18'h001FA;
  assign cosx[4] = 18'h001F6;
  assign cosx[5] = 18'h001F1;
  assign cosx[6] = 18'h001EA;
  assign cosx[7] = 18'h001E2;
  assign cosx[8] = 18'h001D9;
  assign cosx[9] = 18'h001CF;
  assign cosx[10] = 18'h001C4;
  assign cosx[11] = 18'h001B7;
  assign cosx[12] = 18'h001AA;
  assign cosx[13] = 18'h0019B;
  assign cosx[14] = 18'h0018C;
  assign cosx[15] = 18'h0017B;
  assign cosx[16] = 18'h0016A;
  assign cosx[17] = 18'h00158;
  assign cosx[18] = 18'h00145;
  assign cosx[19] = 18'h00131;
  assign cosx[20] = 18'h0011C;
  assign cosx[21] = 18'h00107;
  assign cosx[22] = 18'h000F1;
  assign cosx[23] = 18'h000DB;
  assign cosx[24] = 18'h000C4;
  assign cosx[25] = 18'h000AC;
  assign cosx[26] = 18'h00095;
  assign cosx[27] = 18'h0007C;
  assign cosx[28] = 18'h00064;
  assign cosx[29] = 18'h0004B;
  assign cosx[30] = 18'h00032;
  assign cosx[31] = 18'h00019;
  assign cosx[32] = 18'h00000;
  assign cosx[33] = 18'h3FFE7;
  assign cosx[34] = 18'h3FFCE;
  assign cosx[35] = 18'h3FFB5;
  assign cosx[36] = 18'h3FF9C;
  assign cosx[37] = 18'h3FF84;
  assign cosx[38] = 18'h3FF6B;
  assign cosx[39] = 18'h3FF54;
  assign cosx[40] = 18'h3FF3C;
  assign cosx[41] = 18'h3FF25;
  assign cosx[42] = 18'h3FF0F;
  assign cosx[43] = 18'h3FEF9;
  assign cosx[44] = 18'h3FEE4;
  assign cosx[45] = 18'h3FECF;
  assign cosx[46] = 18'h3FEBB;
  assign cosx[47] = 18'h3FEA8;
  assign cosx[48] = 18'h3FE96;
  assign cosx[49] = 18'h3FE85;
  assign cosx[50] = 18'h3FE74;
  assign cosx[51] = 18'h3FE65;
  assign cosx[52] = 18'h3FE56;
  assign cosx[53] = 18'h3FE49;
  assign cosx[54] = 18'h3FE3C;
  assign cosx[55] = 18'h3FE31;
  assign cosx[56] = 18'h3FE27;
  assign cosx[57] = 18'h3FE1E;
  assign cosx[58] = 18'h3FE16;
  assign cosx[59] = 18'h3FE0F;
  assign cosx[60] = 18'h3FE0A;
  assign cosx[61] = 18'h3FE06;
  assign cosx[62] = 18'h3FE02;
  assign cosx[63] = 18'h3FE01;
  
  always @ (negedge clk_i)
    begin
      if (!rst_i)
        begin
          twr_o <= 18'h0;
          twi_o <= 18'h0;
        end
      else
        begin
          twr_o <= cosx[adr_i];
          twi_o <= sinx[adr_i];
        end
    end
    
endmodule
